// rhsBitmapMonoColoring
// ----------------------------------------------------------------------------
// Example:
// BitmapMonoColoring(Image1.Picture.Bitmap, clRed, 65535, 0, 0, 0.5);
// Intensity 0 - 1
// ----------------------------------------------------------------------------

unit rhsBitmapMonoColoring;

interface

uses
  SysUtils, Classes, Graphics, IntfGraphics, FPImage;

procedure BitmapMonoColoring(BM: TCustomBitmap; AColor: TFPColor; WhiteBorder, GrayBorder, BlackBorder: Word; Intensity: Double);

implementation

function CalcColor(ColorA, ColorB: Integer): Word;
begin
  ColorA := ColorA + ColorB - $3FFF;
  if ColorA < 0 then
    Exit(0);
  if ColorA > $FFFF then
    Exit($FFFF);
  Result := ColorA;
end;

procedure BitmapMonoColoring(BM: TCustomBitmap; AColor: TFPColor; WhiteBorder, GrayBorder, BlackBorder: Word; Intensity: Double);
var
  x, y: Integer;
  IntfImg: TLazIntfImage = nil;
  TempColor: TFPColor;
  NewColor: TFPColor;
  ColorAverage: Word = 0;
begin
  NewColor.Red := Word(Trunc(AColor.Red * Intensity));
  NewColor.Green := Word(Trunc(AColor.Green * Intensity));
  NewColor.Blue := Word(Trunc(AColor.Blue * Intensity));

  try
    IntfImg := BM.CreateIntfImage;

    IntfImg.BeginUpdate;
    for y := 0 to IntfImg.Height - 1 do
      for x := 0 to IntfImg.Width - 1 do
      begin
        TempColor := IntfImg.Colors[x, y];

        ColorAverage := (TempColor.Red + TempColor.Green + TempColor.Blue) div 3;
        if ColorAverage > WhiteBorder then
        begin
          TempColor.Red := CalcColor($12322, NewColor.Red);
          TempColor.Green := CalcColor($12322, NewColor.Green);
          TempColor.Blue := CalcColor($12322, NewColor.Blue);
        end
        else
          if ColorAverage < BlackBorder then
          begin
            TempColor.Red := 0;
            TempColor.Green := 0;
            TempColor.Blue := 0;
          end
          else
            if ColorAverage < GrayBorder then
            begin
              TempColor.Red := CalcColor($3C3C, NewColor.Red);
              TempColor.Green := CalcColor($3C3C, NewColor.Green);
              TempColor.Blue := CalcColor($3C3C, NewColor.Blue);
            end
            else
            begin
              TempColor.Red := CalcColor(ColorAverage, NewColor.Red);
              TempColor.Green := CalcColor(ColorAverage, NewColor.Green);
              TempColor.Blue := CalcColor(ColorAverage, NewColor.Blue);
            end;

        IntfImg.Colors[x, y] := TempColor;
      end;
    IntfImg.EndUpdate;

    BM.LoadFromIntfImage(IntfImg);
  finally
    IntfImg.Free;
  end;
end;

end.
